/* Bluefish HTML Editor
 * html.c - menu/toolbar callbacks, inserting functions, and other cool stuff 
 *
 * Copyright (C) 
 * 1998 Olivier Sessink and Chris Mazuc
 * 1999-2000 Olivier Sessink
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#define DEBUG
#include "default_include.h"

#include <string.h>
#include <locale.h>

#include "cap.h"
#include "bluefish.h"
#include "html.h"
#include "init.h" 	/* positionlist */
#include "gtk_easy.h"
#include "undo.h"
#include "document.h"
#include "stringlist.h"

/*****************************************************************************/
/* function declarations */
void tabledialog_cb(GtkWidget * widget, gpointer data);
void tablerowdialog_cb(GtkWidget * widget, gpointer data);
void tableheaddialog_cb(GtkWidget * widget, gpointer data);
void tabledatadialog_cb(GtkWidget * widget, gpointer data);
void tabledialogok(GtkWidget * widget, gpointer data);
void tablerowdialogok(GtkWidget * widget, gpointer data);
void tableheaddialogok(GtkWidget * widget, gpointer data);
void tabledatadialogok(GtkWidget * widget, gpointer data);

/************************************************************************/
void tabledialog_cb(GtkWidget * widget, gpointer data)
{
	GList *alignlist = NULL;
	GtkWidget *color_but;

	static gchar *tagitems[] = { "cellpadding", "cellspacing", "border", "align", "valign", "bgcolor", "width", NULL };
	gchar *tagvalues[8];
	gchar *custom = NULL;
	Treplacerange *replace_range = NULL;

	/* this is for the right click menu */
	if ((data != NULL) && (widget == NULL)) {
		replace_range = values_for_dialog(tagitems, tagvalues, &custom, (Ttagpopup *) data);
	} else {
		init_values_for_dialog(tagitems, tagvalues);
	}
	/* end for the right click menu */

	create_dialog(500, 125, _("Table"));
	dialogtable = gtk_table_new(4, 12, 0);
	gtk_box_pack_start(GTK_BOX(vbox), dialogtable, FALSE, FALSE, 0);

	spin1 = spinbut_with_value(tagvalues[0], 0, 100, 1.0, 5.0);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), gtk_label_new(_("Cell Padding")), 0, 1, 0, 1);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), spin1, 1, 4, 0, 1);

	alignlist = g_list_append(alignlist, "left");
	alignlist = g_list_append(alignlist, "right");
	alignlist = g_list_append(alignlist, "center");
	combo1 = combo_with_popdown(tagvalues[3], alignlist, 0);
	g_list_free(alignlist);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), gtk_label_new(_("Align")), 4, 5, 0, 1);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), GTK_WIDGET(GTK_COMBO(combo1)), 5, 8, 0, 1);

	spin2 = spinbut_with_value(NULL, 0, 800, 1.0, 5.0);
	checkbox1 = gtk_check_button_new_with_label("%");
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), gtk_label_new(_(" Width ")), 8, 9, 0, 1);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), spin2, 9, 11, 0, 1);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), checkbox1, 11, 12, 0, 1);
	parse_integer_for_dialog(tagvalues[6], spin2, NULL, checkbox1);

	spin3 = spinbut_with_value(tagvalues[1], 0, 100, 1.0, 5.0);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), gtk_label_new(_("Cell Spacing")), 0, 1, 1, 2);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), spin3, 1, 4, 1, 2);

	combo2 = combo_with_popdown(tagvalues[4], positionlist, 1);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), gtk_label_new(_("VAlign")), 4, 5, 1, 2);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), GTK_WIDGET(combo2), 5, 8, 1, 2);

	spin4 = spinbut_with_value(tagvalues[2], 0, 100, 1.0, 5.0);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), gtk_label_new(_("Border")), 8, 9, 1, 2);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), spin4, 9, 12, 1, 2);

	combo3 = combo_with_popdown(tagvalues[5], main_v->current_project.colorlist, 1);
	color_but = color_but_new(GTK_COMBO(combo3)->entry, dialog);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), gtk_label_new(_("Background Color")), 0, 1, 2, 3);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), GTK_WIDGET(combo3), 1, 7, 2, 3);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), GTK_WIDGET(color_but), 7, 8, 2, 3);

	textentry1 = entry_with_text(custom, 1024);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), gtk_label_new(_("Custom")), 0, 1, 3, 4);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), textentry1, 1, 8, 3, 4);
	create_buttons(GTK_SIGNAL_FUNC(tabledialogok), (gpointer) replace_range);
	save_doc_selection();
	if (custom)
		g_free(custom);
}

/************************************************************************/
void tablerowdialog_cb(GtkWidget * widget, gpointer data)
{
	GList *alignlist = NULL;
	GtkWidget *color_but;

	static gchar *tagitems[] = { "align", "valign", "bgcolor", NULL };
	gchar *tagvalues[4];
	gchar *custom = NULL;
	Treplacerange *replace_range = NULL;

	/* this is for the right click menu */
	if ((data != NULL) && (widget == NULL)) {
		replace_range = values_for_dialog(tagitems, tagvalues, &custom, (Ttagpopup *) data);
	} else {
		init_values_for_dialog(tagitems, tagvalues);
	}
	/* end for the right click menu */


	create_dialog(300, 123, _("Table Row"));
	dialogtable = gtk_table_new(3, 10, 0);
	gtk_box_pack_start(GTK_BOX(vbox), dialogtable, FALSE, FALSE, 0);

	alignlist = g_list_insert(alignlist, "left", 0);
	alignlist = g_list_insert(alignlist, "right", 1);
	alignlist = g_list_insert(alignlist, "center", 2);
	combo1 = combo_with_popdown(tagvalues[0], alignlist, 0);
	g_list_free(alignlist);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), gtk_label_new(_("Align")), 0, 1, 0, 1);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), GTK_WIDGET(combo1), 1, 10, 0, 1);

	combo2 = combo_with_popdown(tagvalues[1], positionlist, 0);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), gtk_label_new(_("VAlign")), 0, 1, 1, 2);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), GTK_WIDGET(GTK_COMBO(combo2)), 1, 10, 1, 2);

	combo3 = combo_with_popdown(tagvalues[2], main_v->current_project.colorlist, 1);
	color_but = color_but_new(GTK_COMBO(combo3)->entry, dialog);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), gtk_label_new(_("Background Color")), 0, 1, 2, 3);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), GTK_WIDGET(GTK_COMBO(combo3)), 1, 9, 2, 3);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), GTK_WIDGET(color_but), 9, 10, 2, 3);

	textentry1 = entry_with_text(custom, 1024);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), gtk_label_new(_("Custom")), 0, 1, 3, 4);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), textentry1, 1, 8, 3, 4);
	create_buttons(GTK_SIGNAL_FUNC(tablerowdialogok), (gpointer) replace_range);
	save_doc_selection();
	if (custom)
		g_free(custom);
}

/************************************************************************/
void tableheaddialog_cb(GtkWidget * widget, gpointer data)
{
	GList *alignlist = NULL;

	static gchar *tagitems[] = { "width", "align", "colspan", "height", "valign", "rowspan", "bgcolor", "nowrap", NULL };
	gchar *tagvalues[9];
	gchar *custom = NULL;
	Treplacerange *replace_range = NULL;

	/* this is for the right click menu */
	if ((data != NULL) && (widget == NULL)) {
		replace_range = values_for_dialog(tagitems, tagvalues, &custom, (Ttagpopup *) data);
	} else {
		init_values_for_dialog(tagitems, tagvalues);
	}
	/* end for the right click menu */

	create_dialog(500, 123, _("Table Header"));
	dialogtable = gtk_table_new(3, 12, 0);
	gtk_box_pack_start(GTK_BOX(vbox), dialogtable, FALSE, FALSE, 0);

	spin1 = spinbut_with_value(NULL, 0, 100, 1.0, 5.0);
	checkbox2 = gtk_check_button_new_with_label("%");
	parse_integer_for_dialog(tagvalues[0], spin1, NULL, checkbox2);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), gtk_label_new(_("Width")), 0, 1, 0, 1);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), spin1, 1, 3, 0, 1);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), checkbox2, 3, 4, 0, 1);

	alignlist = g_list_insert(alignlist, "left", 0);
	alignlist = g_list_insert(alignlist, "right", 1);
	alignlist = g_list_insert(alignlist, "center", 2);
	alignlist = g_list_insert(alignlist, "justify", 3);
	combo1 = combo_with_popdown(tagvalues[1], alignlist, 1);
	g_list_free(alignlist);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), gtk_label_new(_("Align")), 4, 5, 0, 1);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), GTK_WIDGET(GTK_COMBO(combo1)), 5, 8, 0, 1);

	spin2 = spinbut_with_value(tagvalues[2], 0, 600, 1.0, 5.0);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), gtk_label_new(_("Col Span")), 8, 9, 0, 1);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), spin2, 9, 12, 0, 1);

	spin3 = spinbut_with_value(tagvalues[3], 0, 100, 1.0, 5.0);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), gtk_label_new(_("Height")), 0, 1, 1, 2);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), spin3, 1, 4, 1, 2);

	combo2 = combo_with_popdown(tagvalues[4], positionlist, 1);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), gtk_label_new(_("VAlign")), 4, 5, 1, 2);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), GTK_WIDGET(GTK_COMBO(combo2)), 5, 8, 1, 2);

	spin4 = spinbut_with_value(tagvalues[5], 0, 100, 1.0, 5.0);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), gtk_label_new(_("Row Span")), 8, 9, 1, 2);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), spin4, 9, 12, 1, 2);

	combo3 = combo_with_popdown(tagvalues[6], main_v->current_project.colorlist, 1);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), gtk_label_new(_("Background Color")), 0, 1, 2, 3);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), GTK_WIDGET(GTK_COMBO(combo3)), 1, 7, 2, 3);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), GTK_WIDGET(color_but_new(GTK_COMBO(combo3)->entry, dialog)), 7, 8, 2, 3);

	checkbox1 = gtk_check_button_new();
	parse_existence_for_dialog(tagvalues[7], checkbox1);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), gtk_label_new(_("No Wrap")), 8, 9, 2, 3);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), checkbox1, 9, 10, 2, 3);

	textentry1 = entry_with_text(custom, 1024);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), gtk_label_new(_("Custom")), 0, 1, 3, 4);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), textentry1, 1, 8, 3, 4);
	create_buttons(GTK_SIGNAL_FUNC(tableheaddialogok), (gpointer) replace_range);
	save_doc_selection();
	if (custom)
		g_free(custom);
}

/************************************************************************/
void tabledatadialog_cb(GtkWidget * widget, gpointer data)
{
	GList *alignlist = NULL;

	static gchar *tagitems[] = { "width", "align", "colspan", "height", "valign", "rowspan", "bgcolor", "nowrap", NULL };
	gchar *tagvalues[9];
	gchar *custom = NULL;
	Treplacerange *replace_range = NULL;

	/* this is for the right click menu */
	if ((data != NULL) && (widget == NULL)) {
		replace_range = values_for_dialog(tagitems, tagvalues, &custom, (Ttagpopup *) data);
	} else {
		init_values_for_dialog(tagitems, tagvalues);
	}
	/* end for the right click menu */

	create_dialog(500, 123, _("Table Data"));
	DEBUG_MSG("tabledatadialog_cb, created\n");
	dialogtable = gtk_table_new(3, 12, 0);
	gtk_box_pack_start(GTK_BOX(vbox), dialogtable, FALSE, FALSE, 0);

	spin1 = spinbut_with_value(NULL, 0, 100, 1.0, 5.0);
	checkbox2 = gtk_check_button_new_with_label("%");
	parse_integer_for_dialog(tagvalues[0], spin1, NULL, checkbox2);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), gtk_label_new(_("Width")), 0, 1, 0, 1);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), spin1, 1, 3, 0, 1);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), checkbox2, 3, 4, 0, 1);

	alignlist = g_list_insert(alignlist, "left", 0);
	alignlist = g_list_insert(alignlist, "right", 1);
	alignlist = g_list_insert(alignlist, "center", 2);
	alignlist = g_list_insert(alignlist, "justify", 3);
	combo1 = combo_with_popdown(tagvalues[1], alignlist, 1);
	g_list_free(alignlist);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), gtk_label_new(_("Align")), 4, 5, 0, 1);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), GTK_WIDGET(GTK_COMBO(combo1)), 5, 8, 0, 1);

	spin2 = spinbut_with_value(tagvalues[2], 0, 10, 1.0, 2.0);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), gtk_label_new(_("Col Span")), 8, 9, 0, 1);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), spin2, 9, 12, 0, 1);

	spin3 = spinbut_with_value(tagvalues[3], 0, 100, 1.0, 5.0);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), gtk_label_new(_("Height")), 0, 1, 1, 2);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), spin3, 1, 4, 1, 2);

	combo2 = combo_with_popdown(tagvalues[4], positionlist, 1);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), gtk_label_new(_("VAlign")), 4, 5, 1, 2);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), GTK_WIDGET(GTK_COMBO(combo2)), 5, 8, 1, 2);
	DEBUG_MSG("tabledatadialog_cb, before spin 4\n");
	spin4 = spinbut_with_value(tagvalues[5], 0, 100, 1.0, 5.0);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), gtk_label_new(_("Row Span")), 8, 9, 1, 2);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), spin4, 9, 12, 1, 2);

	combo3 = combo_with_popdown(tagvalues[6], main_v->current_project.colorlist, 1);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), gtk_label_new(_("Background Color")), 0, 1, 2, 3);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), GTK_WIDGET(GTK_COMBO(combo3)), 1, 7, 2, 3);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), GTK_WIDGET(color_but_new(GTK_COMBO(combo3)->entry, dialog)), 7, 8, 2, 3);

	checkbox1 = gtk_check_button_new();
	parse_existence_for_dialog(tagvalues[7], checkbox1);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), gtk_label_new(_("No Wrap")), 8, 9, 2, 3);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), checkbox1, 9, 10, 2, 3);

	textentry1 = entry_with_text(custom, 1024);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), gtk_label_new(_("Custom")), 0, 1, 3, 4);
	gtk_table_attach_defaults(GTK_TABLE(dialogtable), textentry1, 1, 8, 3, 4);
	DEBUG_MSG("tabledatadialog_cb, about to create buttons\n");
	create_buttons(GTK_SIGNAL_FUNC(tabledatadialogok), (gpointer) replace_range);
	save_doc_selection();
	if (custom)
		g_free(custom);
}

/************************************************************************/
void tabledialogok(GtkWidget * widget, gpointer data)
{
	gchar *thestring;

	thestring = g_strdup(cap("<TABLE"));
	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(spin1)), cap("CELLPADDING"), thestring, NULL);
	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(spin3)), cap("CELLSPACING"), thestring, NULL);
	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(spin4)), cap("BORDER"), thestring, NULL);
	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(GTK_COMBO(combo1)->entry)), cap("ALIGN"), thestring, NULL);
	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(GTK_COMBO(combo2)->entry)), cap("VALIGN"), thestring, NULL);
	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(GTK_COMBO(combo3)->entry)), cap("BGCOLOR"), thestring, NULL);
	thestring = insert_integer_if_spin(spin2, cap("WIDTH"), thestring, checkbox1);
	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(textentry1)), NULL, thestring, NULL);
	stuff = g_strdup_printf("%s>", thestring);
	g_free(thestring);

	if (data == NULL) {
		restore_doc_selection();
		insert_text(stuff, cap("</TABLE>"));
	} else {
		replace_text(stuff, ((Treplacerange *) data)->pos, ((Treplacerange *) data)->len);
		g_free(data);
	}

	destroy_dialog();
}

/************************************************************************/
void tablerowdialogok(GtkWidget * widget, gpointer data)
{
	gchar *thestring;

	thestring = g_strdup(cap("<TR"));
	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(GTK_COMBO(combo1)->entry)), cap("ALIGN"), thestring, NULL);
	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(GTK_COMBO(combo2)->entry)), cap("VALIGN"), thestring, NULL);
	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(GTK_COMBO(combo3)->entry)), cap("BGCOLOR"), thestring, NULL);
	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(textentry1)), NULL, thestring, NULL);
	stuff = g_strdup_printf("%s>", thestring);
	g_free(thestring);

	if (data == NULL) {
		restore_doc_selection();
		insert_text(stuff, cap("</TR>"));
	} else {
		replace_text(stuff, ((Treplacerange *) data)->pos, ((Treplacerange *) data)->len);
		g_free(data);
	}
	destroy_dialog();
}

/************************************************************************/
void tableheaddialogok(GtkWidget * widget, gpointer data)
{
	gchar *thestring;

	thestring = g_strdup(cap("<TH"));
	thestring = insert_integer_if_spin(spin1, cap("WIDTH"), thestring, checkbox2);
	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(spin3)), cap("HEIGHT"), thestring, NULL);
	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(spin2)), cap("COLSPAN"), thestring, NULL);
	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(spin4)), cap("ROWSPAN"), thestring, NULL);
	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(GTK_COMBO(combo1)->entry)), cap("ALIGN"), thestring, NULL);
	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(GTK_COMBO(combo2)->entry)), cap("VALIGN"), thestring, NULL);
	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(GTK_COMBO(combo3)->entry)), cap("BGCOLOR"), thestring, NULL);
	thestring = insert_attr_if_checkbox(checkbox1, cap("NOWRAP"), thestring);
	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(textentry1)), NULL, thestring, NULL);
	stuff = g_strdup_printf("%s>", thestring);
	g_free(thestring);

	if (data == NULL) {
		restore_doc_selection();
		insert_text(stuff, cap("</TH>"));
	} else {
		replace_text(stuff, ((Treplacerange *) data)->pos, ((Treplacerange *) data)->len);
		g_free(data);
	}
	destroy_dialog();
}

/************************************************************************/
void tabledatadialogok(GtkWidget * widget, gpointer data)
{
	gchar *thestring;

	thestring = g_strdup(cap("<TD"));
	thestring = insert_integer_if_spin(spin1, cap("WIDTH"), thestring, checkbox2);
	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(spin3)), cap("HEIGHT"), thestring, NULL);
	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(spin2)), cap("COLSPAN"), thestring, NULL);
	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(spin4)), cap("ROWSPAN"), thestring, NULL);
	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(GTK_COMBO(combo1)->entry)), cap("ALIGN"), thestring, NULL);
	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(GTK_COMBO(combo2)->entry)), cap("VALIGN"), thestring, NULL);
	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(GTK_COMBO(combo3)->entry)), cap("BGCOLOR"), thestring, NULL);
	thestring = insert_attr_if_checkbox(checkbox1, cap("NOWRAP"), thestring);
	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(textentry1)), NULL, thestring, NULL);
	stuff = g_strdup_printf("%s>", thestring);
	g_free(thestring);

	if (data == NULL) {
		restore_doc_selection();
		insert_text(stuff, cap("</TD>"));
	} else {
		replace_text(stuff, ((Treplacerange *) data)->pos, ((Treplacerange *) data)->len);
		g_free(data);
	}
	destroy_dialog();
}
